***********************************************************************
*     file mainstd.F                                                  *
*     initializes coupling parameters for Monte Carlo                 *
*     adapted from Stefan Dittmaier                                   *
*---------------------------------------------------------------------*
*     30.01.06  Ansgar Denner     last changed  22.12.06              *
***********************************************************************


***********************************************************************
*
*	Process pp -> qq'H,   q/q' = any light (anti-)quark
*----------------------------------------------------------------------
* Description of flags:
*
* qborn = 0/1:	switches off/on Born contribution
*
* qw    = 0/1:  switches off/on contributions connected to W exchange in LO
* qz    = 0/1:  switches off/on contributions connected to Z exchange in LO
*
* qschan= 0/1:  switches off/on contributions connected to s-channel W/Z's in LO
* qtchan= 0/1:  switches off/on contributions connected to t-channel W/Z's in LO
*		Note: qschan & qtchan do not work for photon-induced processes,
*		      because this splitting is not gauge invariant there.
*
* qch2  = 0/1:  switches off/on contributions connected to squared LO diagrams
* qchint= 0/1:  switches off/on contributions connected to LO interferences
*
* qbini = 0/1:	switches off/on b-quarks in the initial state
* qbfin = 0/1:	switches off/on b-quarks in the final state
*		Note: channels with external b-quarks included only in LO,
*		      and qbini & qbfin are only active in subroutines
*		      Mat2born, Mat2born1, Mat2born2.
*
* qwidth= 0/1:	switches off/on finite W/Z widths
*		Note: qwidth=1 necessary for complex-mass scheme.
*
* qfact:	internal flag for test purposes
*
* qbos= 0/1/2:	EW bosonic virtual corrections (all but closed fermion loops)
*           0:	not included
*           1:	included in `tHooft-Feynman gauge
*           2:	included in background-field gauge
* qferm = 0/1: 	switches on/off EW corrections from closed fermion loops
* qsoft = 0/1: 	switches on/off soft endpoint part of real corrections
*			in dipole subtraction formalism following
*			S.D., NPB565 (2000) 69
*
* qhh2  = 0/1:	switches on/off 2-loop heavy-Higgs effects of O(GF^2*MH^4)
*			included for squared t-channel diagrams
*			following Frink et al., Phys.Rev.D54 (1996) 4548
*						[hep-ph/9606310]
*
* qqcddiag   =0/1: switches on/off "diagonal" QCD corrections
*			= vertex corrections to squared LO diagrams
*			= category (a) of Bredenstein et al.,
*				JHEP 0702 (2007) 080 [hep-ph/0611234]
* qqcdnondiag=0/1: switches on/off "non-diagonal" QCD corrections
*			= vertex+pentagon corrections to LO interferences
*			= category (b) of Bredenstein et al.
* qqcdgsplit =0/1: switches on/off loops with tree-like g->qqbar splitting
*			= category (c) of Bredenstein et al.
* qqcdggfus  =0/1: switches on/off loop-induced ggH vertex corrections
*			= category (d) of Bredenstein et al.
*
* qcp   = 0/1:	flag (no/yes) for using CP symmetry in amplitudes
*		-> speeds up evaluation of tree diagrams
*
************************************************************************

************************************************************************
        subroutine input
************************************************************************
*	input parameters of the Standard Model
*-----------------------------------------------------------------------
*       20.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        complex*16 guu,gdd
        complex*16 v,cv
	integer n0

        common/param/pi,el,alpha,alpha0,alphaz,GF,alphas,
     &         v(3,3),cv(3,3),cw,cw2,sw,sw2,mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &         ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
     &      ,alphasz
        COMMON/UV/MUDIM2
        COMMON/IR/LAMBDA,LAMBDA2,DELTAS
        common/GEVFB/GEVFB
        COMMON/ALS/XLAMBDA,AMC0,AMB0,AMT0,N0
        common/qf/qu,qd,ql,qn,qf(4),guu(-1:1),gdd(-1:1),mu,mu2,md,md2

	pi    = 4d0*datan(1d0)

        GEVFB = 389379660D0*1000d0

c*** SM parameters from "2002 Review of Particle Physics"
	alpha0  = 1d0/137.03599911d0
	alphaz  = 1d0/128.936d0
c	alphasz = 0.1187d0           ! in readinput
	GF      = 1.16637D-5

	mz      = 91.1876d0
	gz      = 2.4952d0
	mw      = 80.425d0
	gw      = 2.124d0
        mh      = 170d0
        mh      = 120d0
c        mh      = 200d0

cxxx
c	gw=0d0
c	gz=0d0

c transition to pole definition
	wfac = sqrt(1d0+(gw/mw)**2)
	mw   = mw/wfac
	gw   = gw/wfac
	zfac = sqrt(1d0+(gz/mz)**2)
	mz   = mz/zfac
	gz   = gz/zfac

        ml(1)   = .51099892d-3
        ml(2)   = .105658369d0
        ml(3)   = 1.77699d0
        mqm(1)  = .066d0
        mqp(1)  = .066d0
        mqm(2)  = .150d0
        mqp(2)  = 1.2d0
        mqm(3)  = 4.3d0
        mqp(3)  = 174.3d0



c        v(1,1) = 0.975d0
c        v(1,2) = 0.222d0
c        v(2,1) = 0.222d0
c        v(2,2) = 0.975d0

        v(1,1) = 1d0
        v(1,2) = 0d0
        v(2,1) = 0d0
        v(2,2) = 1d0

c	aux    = exp(dcmplx(0d0,0.1d0))
c        v(1,1) = 1d0/2d0                       * aux
c        v(1,2) = sqrt(3d0)/2d0*dcmplx(0d0,1d0) * aux
c        v(2,1) = sqrt(3d0)/2d0*dcmplx(0d0,1d0)
c        v(2,2) = 1d0/2d0

c       v(1,1) = 0.9741d0
c       v(1,2) = sqrt(1d0-v(1,1)*v(1,1))
c       v(2,1) = -v(1,2)
c       v(2,2) = v(1,1)




 	v(1,3) = 0d0
 	v(2,3) = 0d0
 	v(3,1) = 0d0
 	v(3,2) = 0d0
	v(3,3) = 1d0

C*** auxiliary parameters
	mudim2  = mw**2
	lambda  = mw
	lambda2 = lambda**2
	deltas  = 0.1d0

	mu   = mqp(1)

	mu2  = mu**2
	md   = mqm(1)

	md2  = md**2





	end

************************************************************************
        subroutine inputamp
************************************************************************
*	input parameters of the Standard Model for amplitude
*-----------------------------------------------------------------------
*       20.5.99 Stefan Dittmaier       adapted  24.07.07 Ansgar Denner
************************************************************************
        implicit none

#include "mcinput.h"

        real*8 mudim2,lambda,lambda2,deltas,gevfb
        complex*16 v(3,3),cv(3,3)
	integer i1,i2

        real*8 pi,el,alpha,alpha0,alphaz,GF,alphas,
     &         cw,cw2,sw,sw2,mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &         ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
     &        ,alphasz
        common/param/pi,el,alpha,alpha0,alphaz,GF,alphas,
     &         v,cv,cw,cw2,sw,sw2,mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &         ml,ml2,mqp,mqp2,mqm,mqm2
     &        ,alphasz
        COMMON/UV/MUDIM2
        COMMON/IR/LAMBDA,LAMBDA2,DELTAS
        common/GEVFB/GEVFB
        real*8 qu,qd,ql,qn,qf(4),mu,mu2,md,md2
        complex*16 guu(-1:1),gdd(-1:1)
        common/qf/qu,qd,ql,qn,qf,guu,gdd,mu,mu2,md,md2

c initialize independent parameters
        pi    = 4d0*datan(1d0)
        gevfb = gevtofbin

        alpha0  = alpha0in
        alphaz  = alphazin
        GF      = gfin

        alpha   = alphain
        alphas  = alphasin
        alphasz = alphaszin

        mz      = mzin
        gz      = gzin
        mw      = mwin
        gw      = gwin
        mh      = mhin

        ml(1)   = mein
        ml(2)   = mmuin
        ml(3)   = mtauin
        mqm(1)  = mdin
        mqp(1)  = muin
        mqm(2)  = msin
        mqp(2)  = mcin
        mqm(3)  = mbin
        mqp(3)  = mtin

        do i1=1,3
          do i2=1,3
            v(i1,i2) = vin(i1,i2)
          enddo
        enddo

C***  auxiliary parameters
        mudim2  = mudim2in
        lambda  = lambdain
        lambda2 = lambda**2
        deltas  = 0.1d0

        mu   = muin
        mu = mu/1d2
        mu2  = mu**2
        md   = mdin
        md = md/1d2
        md2  = md**2

	end

************************************************************************
        subroutine paraminit
************************************************************************
*	calculate dependent parameters and set auxiliary parameters
*-----------------------------------------------------------------------
*       18.6.02 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        complex*16 v,cv
        complex*16 dalpz,drbos,drferm
	complex*16 ccw,ccw2,csw,csw2,cmw,cmw2,cmz,cmz2
	complex*16 xcw,xcw2,xsw,xsw2,xmw,xmw2,xmz,xmz2,xmh,xmh2,null
	complex*16 xml,xml2,xmqp,xmqp2,xmqm,xmqm2
        complex*16 guu,gdd
        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        integer qborn,qw,qz,qschan,qtchan,qch2,qchint,
     &                   qbini,qbfin,qwidth,qfact,qbos,qferm,qsoft,qhh2,
     &                   qqcddiag,qqcdnondiag,qqcdgsplit,qqcdggfus,qcp
	integer i,j,qalp

        common/param/pi,el,alpha,alpha0,alphaz,GF,alphas,
     &         v(3,3),cv(3,3),cw,cw2,sw,sw2,mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &         ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
     &         ,alphasz
        COMMON/UV/MUDIM2
        COMMON/IR/LAMBDA,LAMBDA2,DELTAS
        common/rscheme/dalpz,drbos,drferm,qalp
	common/cparam/ccw,ccw2,csw,csw2,cmw,cmw2,cmz,cmz2
	common/xparam/xcw,xcw2,xsw,xsw2,xmw,xmw2,xmz,xmz2,xmh,xmh2,null,
     &		      xml(3),xml2(3),xmqp(3),xmqp2(3),xmqm(3),xmqm2(3)
        common/qf/qu,qd,ql,qn,qf(4),guu(-1:1),gdd(-1:1),mu,mu2,md,md2
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2
        common/constsub/constff,constfi,constif,constii
        common/rcoptions/qborn,qw,qz,qschan,qtchan,qch2,qchint,
     &                   qbini,qbfin,qwidth,qfact,qbos,qferm,qsoft,qhh2,
     &                   qqcddiag,qqcdnondiag,qqcdgsplit,qqcdggfus,qcp
	common/hh2/chh2

	cw      = mw/mz
	cw2     = cw**2
	sw2     = 1d0-cw2
	sw      = sqrt(sw2)

        mw2     = mw**2
        mz2     = mz**2
        mh2     = mh**2
	do i=1,3
          ml2(i)  = ml(i)**2
          mqp2(i) = mqp(i)**2
          mqm2(i) = mqm(i)**2
	enddo

C*** renormalization scheme

	el = sqrt(4d0*pi*alpha)

c	alphas = alps(mh,2)

	do i=1,3
	do j=1,3
	  cv(i,j) = dconjg(v(i,j))
	enddo
	enddo

c*** complex parameters
        null = dcmplx(0d0,0d0)
c no widths
	if (qwidth.eq.0) then
          cmw2 = mw2
          cmz2 = mz2
          ccw2 = cw2
          csw2 = sw2
          xmw2 = mw2
          xmz2 = mz2
          xcw2 = cw2
          xsw2 = sw2
c complex-mass scheme
	elseif (qwidth.eq.1) then
          cmw2 = dcmplx(mw2,-mw*gw)
          cmz2 = dcmplx(mz2,-mz*gz)
          ccw2 = cmw2/cmz2
          csw2 = 1d0-ccw2
          xmw2 = cmw2
          xmz2 = cmz2
          xcw2 = ccw2
          xsw2 = csw2
c naive fixed-width scheme
	elseif (qwidth.eq.2) then
          cmw2 = dcmplx(mw2,-mw*gw)
          cmz2 = dcmplx(mz2,-mz*gz)
          ccw2 = cmw2/cmz2
          csw2 = 1d0-ccw2
          xmw2 = mw2
          xmz2 = mz2
          xcw2 = cw2
          xsw2 = sw2
	endif
        cmw  = sqrt(cmw2)
        cmz  = sqrt(cmz2)
        ccw  = sqrt(ccw2)
        csw  = sqrt(csw2)
        xmw  = sqrt(xmw2)
        xmz  = sqrt(xmz2)
        xcw  = sqrt(xcw2)
        xsw  = sqrt(xsw2)

        xmh2 = mh2
        xmh  = sqrt(xmh2)

	do i=1,3
          xml(i)   = ml(i)
          xmqp(i)  = mqp(i)
          xmqm(i)  = mqm(i)
          xml2(i)  = ml2(i)
          xmqp2(i) = mqp2(i)
          xmqm2(i) = mqm2(i)
	enddo

	qu =  2d0/3d0
	qd = -1d0/3d0
	ql = -1d0
	qn =  0d0
        guu(+1) = -qu*xsw/xcw
        guu(-1) = -qu*xsw/xcw+1d0/2d0/xsw/xcw
        gdd(+1) = -qd*xsw/xcw
        gdd(-1) = -qd*xsw/xcw-1d0/2d0/xsw/xcw

        qf(1) = qu
        qf(2) = qd
        qf(3) = qd
        qf(4) = qu

        mf2(1) = mu2
        mf2(2) = md2
        mf2(3) = md2
        mf2(4) = mu2

c fermion identifiers
        do i=1,5
          rmf2(i) = i*1d-20
          cmf2(i) = rmf2(i)
        enddo

c dipole subtraction constants
        constff =  1d0    -pi**2/3d0
        constfi =  1d0    -pi**2/2d0
        constif = -3d0/2d0+pi**2/6d0
        constii =  3d0/2d0-pi**2/3d0

c correction factor for 2-loop heavy-Higgs effects
c taken from Frink et al., Phys.Rev.D54 (1996) 4548 [hep-ph/9606310]
	chh2 = 62.0308d0*(gf*mh2/16d0/pi**2/sqrt(2d0))**2

	end

************************************************************************
        subroutine paramout
************************************************************************
*	output of parameters
*-----------------------------------------------------------------------
*       20.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        complex*16 v,cv
	integer out

        common/param/pi,el,alpha,alpha0,alphaz,GF,alphas,
     &         v(3,3),cv(3,3),cw,cw2,sw,sw2,mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &         ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
     &      ,alphasz
	common/mssm/mue,ma,ma2,mhl,mhl2,mhh,mhh2,mhp,mhp2,
     &	  sb,cb,tb,sa,ca,ta,
     &	  m1,m12,m2,m22,mgl,mgl2,msusy,
     &	  mneu(4),mneu2(4),zneu(4,4),
     &	  mch(2),mch2(2),ucha(2,2),vcha(2,2),
     &	  au(3),ad(3),msq(3),msq2(3),msur(3),msur2(3),msdr(3),msdr2(3),
     &	  ae(3),mse(3),mse2(3),mser(3),mser2(3),
     &	  msqp(3,2),msqp2(3,2),usqp(3,2,2),
     &	  msqm(3,2),msqm2(3,2),usqm(3,2,2),
     &	  msl(3,2),msl2(3,2),usl(3,2,2)

	common/out/out
1000	format(/,'SM parameters:',/,'--------------',/,
     &    2x,'alpha(0) = 1/', f10.6,',',
     &    3x,'alpha(MZ) = 1/',f9.5,',',
     &    3x,'alpha_s = ',f7.5,',',/,
     &    8x,'GF = ',g12.7,',',
     &    26x,'alpha_s(MZ) = ',f7.5,',',/,
     &    8x,'MW = ',f9.5,',',
     &   13x,'MZ = ',f9.5,',',
     &   10x,'MH = ',f9.5,',',/,
     &    8x,'GW = ',f9.5,',',
     &   13x,'GZ = ',f9.5,',',/,
     &    8x,'me = ',g13.8,',',8x,'mmu = ',f13.9,',',
     &    4x,'mtau = ',f9.5,',',/,
     &    8x,'mu = ',f9.5,',',13x,'mc = ',f9.5,',',
     &   10x,'mt = ',f9.5,',',/,
     &    8x,'md = ',f9.5,',',13x,'ms = ',f9.5,',',
     &   10x,'mb = ',f9.5,'.',/)

	write(out,1000) 1d0/alpha0,1d0/alphaz,alphas,GF,alphasz,
     &	  mw,mz,mh,gw,gz,ml(1),ml(2),ml(3),
     &	  mqp(1),mqp(2),mqp(3),mqm(1),mqm(2),mqm(3)


        if( v(1,1).ne.1d0.or.v(2,2).ne.1d0.or.v(3,3).ne.1d0.or.
     &      v(1,2).ne.0d0.or.v(2,3).ne.0d0.or.v(1,3).ne.0d0) then
          write(out,1010) v(1,1),v(1,2),v(1,3),
     &        v(2,1),v(2,2),v(2,3),
     &        v(3,1),v(3,2),v(3,3)
        endif

1010	format(/,
     &    2x,' v(1,1) = (',f8.6,',',f8.6,'),',
     &    2x,' v(1,2) = (',f8.6,',',f8.6,'),',
     &    2x,' v(1,3) = (',f8.6,',',f8.6,'),',/,
     &    2x,' v(2,1) = (',f8.6,',',f8.6,'),',
     &    2x,' v(2,2) = (',f8.6,',',f8.6,'),',
     &    2x,' v(2,3) = (',f8.6,',',f8.6,'),',/,
     &    2x,' v(3,1) = (',f8.6,',',f8.6,'),',
     &    2x,' v(3,2) = (',f8.6,',',f8.6,'),',
     &    2x,' v(3,3) = (',f8.6,',',f8.6,'),',
     &    /)
	end

************************************************************************
	function rho2(a)
************************************************************************
*       AUXILIARY FUNCTION RHO2 FOR THE LEADING TOP-CONTRIBUTIONS      *
*       TO THE RHO-PARAMETER                                           *
*       TAKEN FROM FLEISCHER,TARASOV,JEGERLEHNER PLB319(1993)249       *
*       7.9.94 SD                                                      *
************************************************************************
        IMPLICIT REAL*8 (A-Z)
        COMPLEX*16 CSPEN

        CL(X) = DIMAG( CSPEN( DCMPLX(COS(X),SIN(X)) ) )

	pi = 4d0*datan(1d0)

        IF (A.EQ.4D0) THEN
           F = -4D0*LOG(2D0)
	   G = 0d0
        ELSEIF (A.GT.4D0) THEN
           X = (SQRT(1D0-4D0/A)-1D0)/(SQRT(1D0-4D0/A)+1D0)
           F = -1D0/SQRT(1D0-4D0/A)*(PI**2/6D0+
     &          2D0*DREAL(CSPEN(DCMPLX(X,0D0)))+LOG(-X)**2/2D0 )
           G = SQRT(A-4D0)*LOG(-X)
        ELSE
           F = -2D0/SQRT(4D0/A-1D0)*CL( 2D0*ASIN(SQRT(A/4D0)) )
           G = SQRT(4D0-A)*( PI-2D0*ASIN(SQRT(A/4D0)) )
        ENDIF

        RHO2 = 25D0-4D0*A+(A**2-12D0*A-12D0)*LOG(A)/2D0
     &          +(A-2D0)/2D0/A*PI**2+(A-4D0)/2D0*SQRT(A)*G
     &          -3D0/A*(A-1D0)**2*(A-2D0)
     &                  *DREAL(CSPEN(DCMPLX(1D0-A,0D0)))
     &          +3D0*(A**2-6D0*A+10D0)*F

        END

************************************************************************
        function alphasf(q,loop)
************************************************************************
*       strong running coupling constant                               *
*       q scale                                                        *
*       loop  order of perturbation theory                             *
*----------------------------------------------------------------------*
*       5.1.2007 Stefan Dittmaier  last changed 11.04.07 Ansgar Denner *
************************************************************************
        implicit none
        integer nf,loop
        real*8  q,b1,b2,alphasf,pi,logq
        real*8  lambdalo,lambdanlo

        common/qcd/lambdalo,lambdanlo,nf

        parameter (pi=3.141592653589793238462643383279502884197d0)

        b1 = (33d0-2d0*nf)/(12d0*pi)
        b2 = (153d0-19d0*nf)/(2d0*pi*(33d0-2d0*nf))

        if (loop.eq.1) then
          alphasf = 1d0/(2d0*b1*log(q/lambdalo))
        elseif (loop.eq.2) then
          logq = 2d0*log(q/lambdanlo)
          alphasf = 1d0/(b1*logq)*( 1d0-b2*log(logq)/(b1*logq) )
        else
          write(*,*) 'alphas not supported for loop = ',loop
          stop
        endif

        end

************************************************************************
        subroutine  fixlambdas(q,alphas,loop,n)
************************************************************************
*       determination of Lambfa_QCD from alphas(q)                     *
*       q scale                                                        *
*       alphas  strong coupling at scale q                             *
*       loop  order of perturbation theory                             *
*----------------------------------------------------------------------*
*       11.04.2007 Ansgar Denner   last changed 11.04.07 Ansgar Denner *
************************************************************************
        implicit none
        integer n,nf,loop,i
        real*8  q,alphas
        real*8  b1,b2,pi,h,lambda
        real*8  lambdalo,lambdanlo

        common/qcd/lambdalo,lambdanlo,nf

        parameter (pi=3.141592653589793238462643383279502884197d0)

        nf = n
        b1 = (33d0-2d0*nf)/(12d0*pi)
        b2 = (153d0-19d0*nf)/(2d0*pi*(33d0-2d0*nf))

        if (loop.eq.1) then
          lambdalo = q*exp(-1d0/(2d0*b1*alphas))
c          write(*,*) 'fixla lo ',lambdalo
        elseif (loop.eq.2) then
          h=1d0/alphas+b2*log(b1*alphas/(1d0+b2*alphas))
          lambdanlo=q*exp(-h/(2d0*b1))
c          write(*,*) 'fixla nlo',lambdanlo

          do i=1,30
            h=(1d0-b2*log(2d0*log(q/lambdanlo))
     &          /(b1*2d0*log(q/lambdanlo)))
     &          /alphas
            lambda=q*exp(-h/(2d0*b1))
c            write(*,*) 'fixla nlo',i,lambda

            if(lambda.eq.lambdanlo) return
            lambdanlo=lambda
          enddo
        else
          write(*,*) 'fixlambdas not supported for loop = ',loop
          stop
        endif

        end
